// Multicolor Character TV Snow with Sound
// 30-Aug-2024 by Kevin Krausnick
// Assembler: Kick

.const zp_ptr = $fb
.const sys    = $9e
.const rem    = $8f

* = $801 "Basic"

// basic header
.word line2;.byte 10,0,sys,$20
.for (var x=0;x<4;x++) .byte start.string().charAt(x)
.byte ':',rem,$20;.text "TV SNOW";.byte 0
line2:.word line3;.byte 20,0,rem,$20;.text "30-AUG-2024 BY K. KRAUSNICK";.byte 0
line3:.word line4;.byte 30,0,rem,$20;.text "PRESS 'S' TO TOGGLE SOUND ON/OFF";.byte 0
line4:.word prgend;.byte 40,0,rem,$20;.text "ANY OTHER KEY TO QUIT";.byte 0
prgend:.fill 2,0

start:
  lda #DARK_GRAY
  sta $d020 // border color
  sta $d023 // background #2 color (10 bit pattern)
  lda #GRAY
  sta $d022 // background #1 color (01 bit pattern)
  ldx #BLACK
  stx $d021 // background #0 color (00 bit pattern)
  lda #WHITE | 8 // white (multicolor) (11 bit pattern) in color ram location $d800
  sta $286  // cursor color
  lda #147  // clear screen
  jsr $ffd2

  lda #$f0
  sta $d414 // voice 3 sustain value
  sta $d40f // voice 3 frequency control (high byte)
  lda #$40
  sta $d416 // filter cutoff frequency (high byte)
  lda #$04
  sta $d417 // filter enable voice 3
  lda #$81
  sta $d412 // voice 3 waveform and gate
  lda volume
  sta $d418 // volume and filter select (low pass)
  lda #$1d
  sta $d018 // move character rom address to $3000
  lda #$d8
  sta $d016 // multicolor text mode on

  ldx #8    // 8 pages of custom character ram (256 chars * 8 bytes per char)
  ldy #0
  sty zp_ptr
  lda #$30
  sta zp_ptr+1
!: lda $d41b      // get random number
  sta (zp_ptr),y
  iny
  sta (zp_ptr),y // double height
  iny
  bne !-
  inc zp_ptr+1
  dex
  bne !-

!: lda $d41b // write random characters to screen memory
  sta $400,x
  sta $500,x
  sta $600,x
  sta $6e8,x
  inx
  bne !-
  jsr $ffe4 // until a key is pressed
  beq !-
  cmp #'S'  // toggle sound on/off
  bne !+
  lda volume
  eor #$0f
  sta volume
  sta $d418
  bne !-

!: lda #0    // turn off sound
  sta $d418
  lda #$c8  // restore screen
  sta $d016 // multicolor text mode off
  lda #$15
  sta $d018 // restore character rom address
  lda #WHITE
  sta $286  // cursor color
  lda #147  // clear screen and exit
  jsr $ffd2
  rts

volume: .byte $1f